@@ -4,18 +4,18 @@ use File::Spec ();
 use IO::File   ();
 use vars qw/$AUTOLOAD $VERSION/;
 
-$VERSION = '1.0';
+$VERSION = '1.03';
 
 sub new {
-   my $class   = shift;
-   my $self    = {};
+   my $class = shift;
+   my $self  = {};
    bless $self, $class;
 
       $self->error("Parameters passed to new() must be in 'param => value' format!") if(scalar(@_) % 2);
    my %options = @_; # Parameters;
       $self->validate('new',[qw/-type -path -file -seconds -search -parse_path/],[keys %options]);
 
-   INIT: {
+   INIT:
       $self->{type}          = $options{'-type'}       || 'file'; # file | dir
       $self->{path}          = $options{'-path'}       || '';     # Should never be empty.
       $self->{file}          = $options{'-file'}       || '';     # if type is file, this must be a real file.
@@ -32,9 +32,8 @@ sub new {
       $self->{ACOUNTER}      = 0;     # Counter
       $self->{EXPORT_FORMAT} = undef; # Export to what?
       $self->{ERROR}         = undef; # Contains error messages.
-   }
 
-   CHECK_PARAMS: {
+   CHECK_PARAMS:
       $self->{path} = File::Spec->canonpath($self->{path});
       ($self->{path} and -d $self->{path}) or $self->error("I can't find the directory '$self->{path}': $!");
       if ($self->{type} eq 'file') {
@@ -46,7 +45,6 @@ sub new {
       if ($self->{search_string} and length($self->{search_string}) < 3) {
          $self->error("A search string must be at least three characters long!");
       }
-   }
    return $self;
 }
 
@@ -61,7 +59,7 @@ sub parse {
    #                            'FULLPATH', # always has a value.
    #                           ],
    #                         # Array continues ...
-   #                         ]
+   #                         ],
    #            # and so on ...
    #            };
    #
@@ -121,7 +119,7 @@ sub parse_file {
    # long: total_time, total_songs, total_average_time
    my($ttime,$tsong,$taver) = (0,0,0);
    # while loop variables. j: junk data.
-   my($m3u,$j,$song,$j2,$sec,$temp_sec);
+   my($m3u,$j,@song,$j2,$sec,$temp_sec);
 
       # Open the file to parse:
       $fh->open("< $file") or $self->error("I could't open '$file': $!");
@@ -134,11 +132,11 @@ RECORD:
       next if $m3u =~ m,^#EXTM3U,i;      # First line is just a comment.
       # If the extra information exists, parse it:
       if($m3u =~ m!#EXTINF!i) {
-         ($j ,$song) = split(/\,/,$m3u); # ($artist,$song) = split / - /, $song; ???
+         ($j ,@song) = split(/\,/,$m3u); # ($artist,$song) = split / - /, @song; ???
          ($j ,$sec)  = split(/:/,$j);
          $ttime     += $sec;
          $temp_sec   = $sec;
-         $self->{M3U}{$cd}[$index]->[0] = $song;
+         $self->{M3U}{$cd}[$index]->[0] = join(",", @song);
          if ($sec) {
             $sec = $self->seconds($sec);
          } else {
@@ -174,7 +172,7 @@ RECORD:
                delete $self->{M3U}{$cd}[$index];
             }
          } else {
-            # If we are no searching, just increase the index:
+            # If we are not searching, just increase the index:
             $index++;
          }
          next RECORD;
@@ -232,9 +230,10 @@ sub export {
       my $sc = 0;
          foreach $cd (sort keys %{ $self->{M3U} }) {
             $sc = $#{$self->{M3U}{$cd}}+1;
+            next unless $sc;
             print $fh qq~<cd name="$cd" drive="$self->{DRIVE}{$cd}" songs="$sc">\n~;
             foreach $m3u (sort @{ $self->{M3U}{$cd} }) {
-               print $fh sprintf qq~<song id3="%s" time="%s">%s</song>\n~,$self->escape($m3u->[0]),$m3u->[1],$self->escape($m3u->[2]);
+               print $fh sprintf qq~<song id3="%s" time="%s">%s</song>\n~,$self->escape($m3u->[0]) || '',$m3u->[1] || '',$self->escape($m3u->[2]);
             }
             print $fh "</cd>\n";
             $sc = 0;
@@ -252,7 +251,8 @@ sub export {
          print $fh $self->html('start',total => $self->{TOTAL_FILES},
                                        ttime => $self->{TOTAL_TIME},
                                        songs => $self->{TOTAL_SONGS},
-                                       file  => $file);
+                                       file  => $file,
+                                       encod => $encoding);
          my $song;
          my $cdrom;
          foreach $cd (sort keys %{ $self->{M3U} }) {
@@ -337,7 +337,7 @@ sub read_dir {
       next unless /\.m3u$/i;
       push @list, File::Spec->catfile($path,$_);
    }
-#   File::Find::find(sub {return unless /\.m3u$/i; push @list, File::Spec->catfile($path,$_)}, $path);
+
    return(sort @list);
 }
 
@@ -345,7 +345,7 @@ sub seconds {
    # Format seconds if wanted.
    my $self = shift;
    my $all  = shift || 1;
-   return($all) unless( $self->{seconds} eq 'format' );
+   return($all) unless( $self->{seconds} eq 'format' and $all !~ /:/);
       $all  = $all/60;
    my $min  = int($all);
    my $sec  = sprintf("%02d",int(($all - $min)*60));
@@ -405,7 +405,7 @@ sub validate {
    my %known  = map {$_ => 1} @{+shift};
    my %passed = map {$_ => 1} @{+shift};
    foreach (keys %passed) {
-      $self->error("Validate: Unknown parameter '$_' passed to $sub()!") unless(exists $known{$_});
+      $self->error("Unknown parameter '$_' passed to $sub()!") unless(exists $known{$_});
    }
 }
 
@@ -449,80 +449,98 @@ sub html {
       }
    }
 
-   unless($self->{AVERAGE_TIME}) {
-      $self->{AVERAGE_TIME} = '<i>Unknown</i>';
+   my $avertime;
+   if($self->{AVERAGE_TIME}) {
+      $avertime = $self->seconds($self->{AVERAGE_TIME});
+   } else {
+      $avertime = '<i>Unknown</i>';
    }
 
    if ($type eq 'start') {
    my $pls = ($self->{TOTAL_FILES} > 1) ? "Playlists and Files" : "Playlist files";
 # Based on WinAmp' s HTML List:
 return <<HTML_START;
+<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
+   "http://www.w3.org/TR/html4/loose.dtd">
 <html>
  <head>
+
    <title>MP3::M3U::Parser Generated PlayList</title>
+
+   <meta name="Generator" content="MP3::M3U::Parser">
+   <meta http-equiv="content-type" content="text/html; charset=$opt{'encod'}">
+
    <style type="text/css">
-<!--
-  BODY { background   : #000040; }
-.para1 { margin-top   : -42px;
-         margin-left  : 350px;
-         margin-right : 10px;
-         font-family  : "font2, Arial";
-         font-size    : 30px; line-height: 35px;
-         text-align   : left;
-         color        : #E1E1E1;
-         }
-.para2 { margin-top   : 15px;
-         margin-left  : 15px;
-         margin-right : 50px;
-         font-family  : "font1, Arial Black";
-         font-size    : 50px;
-         line-height  : 40px;
-         text-align   : left;
-         color        : #004080;
-         }
-td { font-family : "Arial";
-     color       : "#FFFFFF";
-     font-size   : 13px;
-     }
-.t { font-family : "Arial";
-     color       : "#FFBF00";
-     font-size   : 13px;
-     }
-.ts {font-family : "Arial";
-     color       : "#FFBF00";
-     font-size   : 10px;
-     }
-.s { font-family : "Arial";
-     color       : "#FFFFFF";
-     font-size   : 13px;
-     }
-.info {
-     font-family : "Arial";
-     color       : "#409FFF";
-     font-size   : 10px;
-      }
-.infobig {
-     font-family : "Arial";
-     color       : "#FFBF00";
-     font-size   : 15px;
-      }
--->
+   <!--
+      body   { background   : #000040;
+               font-family  : font1, Arial, serif;
+               color        : #FFFFFF;
+               font-size    : 10pt;    }
+      td     { background   : none;
+               font-family  : Arial, serif;
+               color        : #FFFFFF;
+               font-size    : 13px;    }
+      hr     { background   : none;
+               color        : #FFBF00; }
+      .para1 { margin-top   : -42px;
+               margin-left  : 350px;
+               margin-right : 10px;
+               font-family  : font2, Arial, serif;
+               font-size    : 30px; 
+               line-height  : 35px;
+               background   : none;
+               color        : #E1E1E1;
+               text-align   : left;    }
+      .para2 { margin-top   : 15px;
+               margin-left  : 15px;
+               margin-right : 50px;
+               font-family  : font1, Arial Black, serif;
+               font-size    : 50px;
+               line-height  : 40px;
+               background   : none;
+               color        : #004080;
+               text-align   : left;    }
+      .t     { font-family  : Arial, serif;
+               background   : none;
+               color        : #FFBF00;
+               font-size    : 13px;    }
+      .ts    { font-family  : Arial, serif;
+               color        : #FFBF00;
+               background   : none;
+               font-size    : 10px;    }
+      .s     { font-family  : Arial, serif;
+               background   : none;
+               color        : #FFFFFF;
+               font-size    : 13px;    }
+      .info  { font-family  : Arial, serif;
+               background   : none;
+               color        : #409FFF;
+               font-size    : 10px;    }
+      .infob { font-family  : Arial, serif;
+               background   : none;
+               color        : #FFBF00;
+               font-size    : 15px;    }
+    -->
    </style>
+
  </head>
-<body BGCOLOR="#000080" topmargin="0" leftmargin="0" text="#FFFFFF">
+
+<body>
+
  <div align="center">
-  <div CLASS="para2" align="center"><p>MP3::M3U::Parser</p></div>
-  <div CLASS="para1" align="center"><p>playlist</p></div>
+  <div class="para2" align="center"><p>MP3::M3U::Parser</p></div>
+  <div class="para1" align="center"><p>playlist</p></div>
  </div>
 
-<hr align="left" width="90%" noshade size="1" color="#FFBF00">
- <div align="right">
+<hr align="left" width="90%" noshade="noshade" size="1">
+ <div align="left">
+
   <table border="0" cellspacing="0" cellpadding="0" width="98%">
    <tr><td>
     <span class="ts">$opt{songs}</span> <span class="info"> tracks and 
     <span class="ts">$opt{total}</span> CDs in playlist, 
       average track length: </span> 
-      <span class="ts">$self->{AVERAGE_TIME}</span><span class="info">.</span>
+      <span class="ts">$avertime</span><span class="info">.</span>
      <br>
     <span class="info">Playlist length: </span> 
      $time
@@ -532,9 +550,10 @@ td { font-family : "Arial";
     </td>
    </tr>
  </table>
+
 </div>
 <blockquote>
-<p><span class="infobig"><big>$pls:</big></span></p>
+<p><span class="infob"><big>$pls:</big></span></p>
 
 <table border="0" cellspacing="1" cellpadding="2">
 
@@ -544,7 +563,7 @@ HTML_START
       return <<HTML_END;
   </table>
 </blockquote>
-<hr align="left" width="90%" noshade size="1" color="#FFBF00">
+<hr align="left" width="90%" noshade size="1">
 <span class="s">This HTML File is based on 
 <a href="http://www.winamp.com">WinAmp</a>'s HTML List.</span>
 </body>
@@ -563,11 +582,12 @@ sub AUTOLOAD {
 }
 
 sub DESTROY {
-   my $self = shift;
+#   my $self = shift;
 #   delete $self->{$_} foreach keys %$self;
 }
 
 1;
+
 __END__;
 
 =head1 NAME
@@ -649,7 +669,7 @@ Else: you get the time in seconds like; I<256> (if formatted: I<04:15>).
 
 If you don't want to get a list of every song in the m3u list, but want to get 
 a specific group's/singer's songs from the list, set this to the string you want 
-to search. 
+to search. Think this "search" as a parser filter.
 
 Note that, the module will do a *very* basic case-insensitive search. It does 
 dot accept multiple words (if you pass a string like "michael beat it", it will 
@@ -657,13 +677,14 @@ not search every word seperated by space, it will search the string "michael bea
 and probably does not return any results -- it will not match 
 "michael jackson - beat it"), it does not have a boolean search support, etc. If you 
 want to do something more complex, get the parsed tree and use it in your own 
-search function.
+search function, or just re-define the private method C<search> like the private 
+method L<html|/"How can I change the HTML List layout?">.
 
 =item C<-parse_path>
 
 The module assumes that all of the songs in your M3U lists are (or were: 
 the module does not check the existence of them) on the same drive. And it 
-builds a seperate table for drive names like:
+builds a seperate data table for drive names like:
 
    DRIVE => {
              FIRST_LIST => 'G:',
@@ -709,6 +730,33 @@ or
 
    my %hash = $parser->parse;
 
+The data returned from C<parse()> method is like this (lets say, we are 
+parsing a list named 'I<mp3_16.m3u>'):
+
+   # Parse the list(s) and get results:
+   $results = $parser->parse;
+
+   # $results is like this:
+   $results = {
+     'mp3_16' => [ # The parsed list's name is the main key.
+                   # and it's value is an arrayref.
+                   [ # inside, the values are also arrayrefs.
+                    '2 Unlimited - No Limit',          # 1st: ID3 Name
+                    '03:29',                           # 2nd: Time
+                    'G:\EN\2 Unlimited - No limit.mp3' # 3rd: File path.
+                   ],
+                   # If there are other songs in the list, they'll be added here.
+                 ],# End of 'mp3_16' key.
+    # if we've parsed other lists, their content is here.        
+   };
+
+You can also use 
+
+   my $results = $parser->parse;
+      $parser->Dump("/path/to/mydump.txt",[$results]);
+
+to see the structure yourself.
+
 =item B<info()>
 
 You must call this after calling L<parse|/parse()>. It returns an info hash about 
@@ -722,7 +770,7 @@ The keys of the C<%info> hash are:
    files   => Total number of lists parsed
    ttime   => Total time of the songs 
    average => Average time of the songs
-   drive   => Drive manes for parsed lists
+   drive   => Drive names for parsed lists
 
 Note that the 'drive' key is a hash ref, while others are strings. If you 
 have parsed a M3U list named "mp3_01.m3u" and want to learn the drive letter 
@@ -753,7 +801,10 @@ Can be C<xml> or C<html>. Default is C<html>.
 =item C<-encoding>
 
 The exported C<xml> file's encoding. Default is B<ISO-8859-1>. 
-See L<http://www.iana.org/assignments/character-sets> for a list.
+See L<http://www.iana.org/assignments/character-sets> for a list. 
+If you don't define the correct encoding for xml, you can get 
+"not well-formed" errors from the xml parsers. This value is 
+also used in the meta tag section of the html file.
 
 =item C<-drives>
 
@@ -768,9 +819,9 @@ C<on>. Also see L<parse_path|/-parse_path> option in L<new|/new()>.
 Dumps the data structure to a text file. Call this with a data structure 
 and full path of the text file you want to save:
 
-   $parser->Dump([%results],"/my/dump/file.txt");
+   $parser->Dump("/my/dump/file.txt",[%results]);
 
-First argument must be an arrayref. Uses the standard Data::Dumper module.
+Second argument must be an arrayref. Uses the standard Data::Dumper module.
 
 =back
 
@@ -784,7 +835,7 @@ using C<eval> for all method calls can be helpful if you don't want to die:
       my %results;
       eval { %results = $parser->parse };
       if($@) {
-         warn "There was an error: $parser->{ERROR}";
+         warn "There was an error: $@";
          return;
       } else {
          # do something with %results ...
@@ -793,7 +844,7 @@ using C<eval> for all method calls can be helpful if you don't want to die:
    }
 
 As you can see, if there is an error, you can catch this with C<eval> and 
-access the error message with the special object table called C<ERROR>.
+access the error message with the special Perl variable C<$@>.
 
 =head1 EXAMPLES
 
@@ -841,7 +892,8 @@ Solution: As you can see in the above examples:
 
       my @lists = keys %results;
 
-Note that the C<.m3u> extension is removed from the file names when parsing.
+Note that the C<.m3u> extension and path information are removed from the file 
+names when parsing.
 
 =item B<How can I "just" export the parsed tree to a format?>
 
@@ -853,10 +905,10 @@ Solution (convert to xml and save):
                          -path    => '/my/path',
                          -seconds => 'format',
                          )
-                         ->parse(1)
-                         ->export(-format   => 'xml',
-                                  -file     => "/my/export/path/mp3.xml",
-                                  -encoding => 'ISO-8859-9');
+                   ->parse(1)
+                   ->export(-format   => 'xml',
+                            -file     => "/my/export/path/mp3.xml",
+                            -encoding => 'ISO-8859-9');
    print "Done!\n";
 
 If you dont need any objects, just call new() followed by parse() followed by 
@@ -898,6 +950,8 @@ because it belongs to your name space not this module's. Then change what
 you want. I don't plan to add a template option, because I find it unnecessary. 
 But if you want to do such a thing, this is the solution.
 
+Note: re-definitions generate warnings.
+
 =back
 
 =head1 NOTES
@@ -907,8 +961,6 @@ M3U parser in CPAN, but it does not have the functionality I want and it skips
 the EXTINFO lines. So, I wrote this module. This module does not check the 
 existence of the mp3 files, it just parses the playlist entries.
 
-Tested under MS Win98/2000 (Active Perl 5.8.0 Build 804) and RH Linux 8.0 (Perl 5.8.0).
-
 =head2 TIPS
 
 =over 4
@@ -938,10 +990,6 @@ you can have an easy maintained archive.
 
 Better xml transformation.
 
-=item * 
-
-Add C<flock()> (maybe).
-
 =item *
 
 Split big exported(HTML) into separate files (maybe).
